// This file is autogenerated. Changes will be overwritten.
#include "GCU4LA4VMI/moc_idlewatcher.cpp"
#include "GCU4LA4VMI/moc_idlewatcher_wl.cpp"
#include "EWIEGA46WW/moc_lidswitchhandler.cpp"
#include "EWIEGA46WW/moc_lowpowermanager.cpp"
#include "EWIEGA46WW/moc_powermanager.cpp"
#include "EWIEGA46WW/moc_powersaveplan.cpp"
#include "JGIV4DAHX4/moc_screencontroller.cpp"
#include "JGIV4DAHX4/moc_screencontroller_wl.cpp"
#include "EWIEGA46WW/moc_sessiondbusproxy.cpp"
#include "EWIEGA46WW/moc_sleepinhibitor.cpp"
