/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024-2025 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface treeland_shortcut_capture_v1_interface;
extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *treeland_shortcut_manager_v2_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	&wl_seat_interface,
	&treeland_shortcut_capture_v1_interface,
};

static const struct wl_message treeland_shortcut_manager_v2_requests[] = {
	{ "destroy", "", treeland_shortcut_manager_v2_types + 0 },
	{ "acquire", "", treeland_shortcut_manager_v2_types + 0 },
	{ "bind_key", "ssuu", treeland_shortcut_manager_v2_types + 0 },
	{ "bind_swipe_gesture", "suuu", treeland_shortcut_manager_v2_types + 0 },
	{ "bind_hold_gesture", "suu", treeland_shortcut_manager_v2_types + 0 },
	{ "commit", "", treeland_shortcut_manager_v2_types + 0 },
	{ "unbind", "s", treeland_shortcut_manager_v2_types + 0 },
	{ "capture_next_shortcut", "2o?on", treeland_shortcut_manager_v2_types + 4 },
};

static const struct wl_message treeland_shortcut_manager_v2_events[] = {
	{ "activated", "su", treeland_shortcut_manager_v2_types + 0 },
	{ "commit_success", "", treeland_shortcut_manager_v2_types + 0 },
	{ "commit_failure", "su", treeland_shortcut_manager_v2_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_shortcut_manager_v2_interface = {
	"treeland_shortcut_manager_v2", 3,
	8, treeland_shortcut_manager_v2_requests,
	3, treeland_shortcut_manager_v2_events,
};

static const struct wl_message treeland_shortcut_capture_v1_requests[] = {
	{ "destroy", "", treeland_shortcut_manager_v2_types + 0 },
};

static const struct wl_message treeland_shortcut_capture_v1_events[] = {
	{ "captured", "s", treeland_shortcut_manager_v2_types + 0 },
	{ "failed", "u", treeland_shortcut_manager_v2_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_shortcut_capture_v1_interface = {
	"treeland_shortcut_capture_v1", 1,
	1, treeland_shortcut_capture_v1_requests,
	2, treeland_shortcut_capture_v1_events,
};

