/****************************************************************************
** Meta object code from reading C++ file 'screencontroller_wl.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../screen/screencontroller_wl.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'screencontroller_wl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11OutputPowerE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputPower::qt_create_metaobjectdata<qt_meta_tag_ZN11OutputPowerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputPower",
        "modeChanged",
        "",
        "uint32_t",
        "mode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OutputPower, qt_meta_tag_ZN11OutputPowerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OutputPower::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11OutputPowerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11OutputPowerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11OutputPowerE_t>.metaTypes,
    nullptr
} };

void OutputPower::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputPower *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OutputPower::*)(uint32_t )>(_a, &OutputPower::modeChanged, 0))
            return;
    }
}

const QMetaObject *OutputPower::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputPower::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11OutputPowerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::zwlr_output_power_v1"))
        return static_cast< QtWayland::zwlr_output_power_v1*>(this);
    return QObject::qt_metacast(_clname);
}

int OutputPower::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OutputPower::modeChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN18OutputPowerManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputPowerManager::qt_create_metaobjectdata<qt_meta_tag_ZN18OutputPowerManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputPowerManager"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OutputPowerManager, qt_meta_tag_ZN18OutputPowerManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OutputPowerManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<OutputPowerManager>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputPowerManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputPowerManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18OutputPowerManagerE_t>.metaTypes,
    nullptr
} };

void OutputPowerManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputPowerManager *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OutputPowerManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputPowerManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputPowerManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::zwlr_output_power_manager_v1"))
        return static_cast< QtWayland::zwlr_output_power_manager_v1*>(this);
    return QWaylandClientExtensionTemplate<OutputPowerManager>::qt_metacast(_clname);
}

int OutputPowerManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<OutputPowerManager>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18OutputColorControlE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputColorControl::qt_create_metaobjectdata<qt_meta_tag_ZN18OutputColorControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputColorControl",
        "brightnessChanged",
        "",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'brightnessChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OutputColorControl, qt_meta_tag_ZN18OutputColorControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OutputColorControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputColorControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputColorControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18OutputColorControlE_t>.metaTypes,
    nullptr
} };

void OutputColorControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputColorControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->brightnessChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OutputColorControl::*)(double )>(_a, &OutputColorControl::brightnessChanged, 0))
            return;
    }
}

const QMetaObject *OutputColorControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputColorControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18OutputColorControlE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_output_color_control_v1"))
        return static_cast< QtWayland::treeland_output_color_control_v1*>(this);
    return QObject::qt_metacast(_clname);
}

int OutputColorControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OutputColorControl::brightnessChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN17TreeLandOutputMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto TreeLandOutputMgr::qt_create_metaobjectdata<qt_meta_tag_ZN17TreeLandOutputMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreeLandOutputMgr"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeLandOutputMgr, qt_meta_tag_ZN17TreeLandOutputMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreeLandOutputMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<TreeLandOutputMgr>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TreeLandOutputMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TreeLandOutputMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17TreeLandOutputMgrE_t>.metaTypes,
    nullptr
} };

void TreeLandOutputMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeLandOutputMgr *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TreeLandOutputMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreeLandOutputMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TreeLandOutputMgrE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_output_manager_v1"))
        return static_cast< QtWayland::treeland_output_manager_v1*>(this);
    return QWaylandClientExtensionTemplate<TreeLandOutputMgr>::qt_metacast(_clname);
}

int TreeLandOutputMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<TreeLandOutputMgr>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23WaylandScreenControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto WaylandScreenController::qt_create_metaobjectdata<qt_meta_tag_ZN23WaylandScreenControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WaylandScreenController"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WaylandScreenController, qt_meta_tag_ZN23WaylandScreenControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WaylandScreenController::staticMetaObject = { {
    QMetaObject::SuperData::link<ScreenController::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23WaylandScreenControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23WaylandScreenControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23WaylandScreenControllerE_t>.metaTypes,
    nullptr
} };

void WaylandScreenController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WaylandScreenController *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *WaylandScreenController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WaylandScreenController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23WaylandScreenControllerE_t>.strings))
        return static_cast<void*>(this);
    return ScreenController::qt_metacast(_clname);
}

int WaylandScreenController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScreenController::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
