/****************************************************************************
** Meta object code from reading C++ file 'systemgestureproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/backend/x11/systemgestureproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'systemgestureproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18SystemGestureProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto SystemGestureProxy::qt_create_metaobjectdata<qt_meta_tag_ZN18SystemGestureProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SystemGestureProxy",
        "eventReceived",
        "",
        "name",
        "direction",
        "fingers",
        "doubleClickDown",
        "swipeMoving",
        "accelX",
        "accelY",
        "swipeStopped",
        "availabilityChanged",
        "available",
        "onServiceRegistered",
        "onServiceUnregistered",
        "onEvent",
        "onDoubleClickDown",
        "onSwipeMoving",
        "onSwipeStopped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'eventReceived'
        QtMocHelpers::SignalData<void(const QString &, const QString &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'doubleClickDown'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'swipeMoving'
        QtMocHelpers::SignalData<void(int, double, double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 },
        }}),
        // Signal 'swipeStopped'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'availabilityChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'onServiceRegistered'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onServiceUnregistered'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onEvent'
        QtMocHelpers::SlotData<void(const QString &, const QString &, int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'onDoubleClickDown'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'onSwipeMoving'
        QtMocHelpers::SlotData<void(int, double, double)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 },
        }}),
        // Slot 'onSwipeStopped'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SystemGestureProxy, qt_meta_tag_ZN18SystemGestureProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SystemGestureProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SystemGestureProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SystemGestureProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18SystemGestureProxyE_t>.metaTypes,
    nullptr
} };

void SystemGestureProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SystemGestureProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->eventReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 1: _t->doubleClickDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->swipeMoving((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 3: _t->swipeStopped((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->availabilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->onServiceRegistered(); break;
        case 6: _t->onServiceUnregistered(); break;
        case 7: _t->onEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->onDoubleClickDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->onSwipeMoving((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 10: _t->onSwipeStopped((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SystemGestureProxy::*)(const QString & , const QString & , int )>(_a, &SystemGestureProxy::eventReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SystemGestureProxy::*)(int )>(_a, &SystemGestureProxy::doubleClickDown, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SystemGestureProxy::*)(int , double , double )>(_a, &SystemGestureProxy::swipeMoving, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SystemGestureProxy::*)(int )>(_a, &SystemGestureProxy::swipeStopped, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SystemGestureProxy::*)(bool )>(_a, &SystemGestureProxy::availabilityChanged, 4))
            return;
    }
}

const QMetaObject *SystemGestureProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SystemGestureProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SystemGestureProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SystemGestureProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void SystemGestureProxy::eventReceived(const QString & _t1, const QString & _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void SystemGestureProxy::doubleClickDown(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SystemGestureProxy::swipeMoving(int _t1, double _t2, double _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void SystemGestureProxy::swipeStopped(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SystemGestureProxy::availabilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
