// SPDX-FileCopyrightText: 2026 UnionTech Software Technology Co., Ltd.
// SPDX-License-Identifier: LGPL-3.0-or-later

#include "ambientbrightnesslogging.h"

Q_LOGGING_CATEGORY(logAmbientBrightness, "dde.ambientbrightness")
