/****************************************************************************
** Meta object code from reading C++ file 'ambientbrightnessservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../ambientbrightnessservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ambientbrightnessservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto dde::ambient_brightness::AmbientBrightnessService::qt_create_metaobjectdata<qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dde::ambient_brightness::AmbientBrightnessService",
        "D-Bus Interface",
        "org.deepin.dde.AmbientBrightness1",
        "supportedChanged",
        "",
        "value",
        "stateChanged",
        "enabledChanged",
        "recommendedBrightnessChanged",
        "Enable",
        "active",
        "onSensorServiceRegistered",
        "onSensorServiceUnregistered",
        "onLidClosed",
        "onLidOpened",
        "onPrepareForSleep",
        "beforeSleep",
        "onSessionActiveChanged",
        "onSessionPropertiesChanged",
        "interface",
        "QVariantMap",
        "changed",
        "onAutomaticBrightnessEnabledChanged",
        "enabled",
        "onPropertiesChanged",
        "invalidated",
        "Supported",
        "State",
        "Enabled",
        "RecommendedBrightness"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'supportedChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'recommendedBrightnessChanged'
        QtMocHelpers::SignalData<void(double)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'Enable'
        QtMocHelpers::SlotData<void(bool)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'onSensorServiceRegistered'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSensorServiceUnregistered'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onLidClosed'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onLidOpened'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPrepareForSleep'
        QtMocHelpers::SlotData<void(bool)>(15, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'onSessionActiveChanged'
        QtMocHelpers::SlotData<void(bool)>(17, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'onSessionPropertiesChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &, const QStringList &)>(18, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { 0x80000000 | 20, 21 }, { QMetaType::QStringList, 4 },
        }}),
        // Slot 'onAutomaticBrightnessEnabledChanged'
        QtMocHelpers::SlotData<void(bool)>(22, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'onPropertiesChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &, const QStringList &)>(24, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { 0x80000000 | 20, 21 }, { QMetaType::QStringList, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Supported'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'State'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'Enabled'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'RecommendedBrightness'
        QtMocHelpers::PropertyData<double>(29, QMetaType::Double, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<AmbientBrightnessService, qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject dde::ambient_brightness::AmbientBrightnessService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>.metaTypes,
    nullptr
} };

void dde::ambient_brightness::AmbientBrightnessService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AmbientBrightnessService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->supportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->enabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->recommendedBrightnessChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->Enable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->onSensorServiceRegistered(); break;
        case 6: _t->onSensorServiceUnregistered(); break;
        case 7: _t->onLidClosed(); break;
        case 8: _t->onLidOpened(); break;
        case 9: _t->onPrepareForSleep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->onSessionActiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->onSessionPropertiesChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 12: _t->onAutomaticBrightnessEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->onPropertiesChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AmbientBrightnessService::*)(bool )>(_a, &AmbientBrightnessService::supportedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmbientBrightnessService::*)(const QString & )>(_a, &AmbientBrightnessService::stateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmbientBrightnessService::*)(bool )>(_a, &AmbientBrightnessService::enabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmbientBrightnessService::*)(double )>(_a, &AmbientBrightnessService::recommendedBrightnessChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->supported(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->state(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->recommendedBrightness(); break;
        default: break;
        }
    }
}

const QMetaObject *dde::ambient_brightness::AmbientBrightnessService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dde::ambient_brightness::AmbientBrightnessService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde18ambient_brightness24AmbientBrightnessServiceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dde::ambient_brightness::AmbientBrightnessService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void dde::ambient_brightness::AmbientBrightnessService::supportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void dde::ambient_brightness::AmbientBrightnessService::stateChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void dde::ambient_brightness::AmbientBrightnessService::enabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dde::ambient_brightness::AmbientBrightnessService::recommendedBrightnessChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
