/****************************************************************************
** Meta object code from reading C++ file 'sessiondbusproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../sessiondbusproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sessiondbusproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SessionDBusProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto SessionDBusProxy::qt_create_metaobjectdata<qt_meta_tag_ZN16SessionDBusProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SessionDBusProxy",
        "OnBatteryChanged",
        "",
        "value",
        "HasLidSwitchChanged",
        "HasBatteryChanged",
        "BatteryPercentageChanged",
        "BatteryStatusChanged",
        "BatteryTimeToEmptyChanged",
        "IsHighPerformanceSupportedChanged",
        "PowerSavingModeEnabledChanged",
        "PowerSavingModeBrightnessDropPercentChanged",
        "SessionActiveChanged",
        "notifyActionInvoked",
        "id",
        "actionKey",
        "timeUpdate",
        "login1OwnerChanged",
        "name",
        "oldOwner",
        "newOwner",
        "OnBattery",
        "HasLidSwitch",
        "HasBattery",
        "batteryPercentage",
        "batteryStatus",
        "batteryTimeToEmpty",
        "IsHighPerformanceSupported",
        "PowerSavingModeEnabled",
        "PowerSavingModeBrightnessDropPercent",
        "SessionActive"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'OnBatteryChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'HasLidSwitchChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'HasBatteryChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'BatteryPercentageChanged'
        QtMocHelpers::SignalData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BatteryStatusChanged'
        QtMocHelpers::SignalData<void(uint)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 },
        }}),
        // Signal 'BatteryTimeToEmptyChanged'
        QtMocHelpers::SignalData<void(quint64)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 3 },
        }}),
        // Signal 'IsHighPerformanceSupportedChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'PowerSavingModeEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'PowerSavingModeBrightnessDropPercentChanged'
        QtMocHelpers::SignalData<void(uint)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 },
        }}),
        // Signal 'SessionActiveChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'notifyActionInvoked'
        QtMocHelpers::SignalData<void(uint, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 14 }, { QMetaType::QString, 15 },
        }}),
        // Signal 'timeUpdate'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'login1OwnerChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'OnBattery'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'HasLidSwitch'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'HasBattery'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'batteryPercentage'
        QtMocHelpers::PropertyData<double>(24, QMetaType::Double, QMC::DefaultPropertyFlags, 3),
        // property 'batteryStatus'
        QtMocHelpers::PropertyData<uint>(25, QMetaType::UInt, QMC::DefaultPropertyFlags, 4),
        // property 'batteryTimeToEmpty'
        QtMocHelpers::PropertyData<quint64>(26, QMetaType::ULongLong, QMC::DefaultPropertyFlags, 5),
        // property 'IsHighPerformanceSupported'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'PowerSavingModeEnabled'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'PowerSavingModeBrightnessDropPercent'
        QtMocHelpers::PropertyData<uint>(29, QMetaType::UInt, QMC::DefaultPropertyFlags, 8),
        // property 'SessionActive'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SessionDBusProxy, qt_meta_tag_ZN16SessionDBusProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SessionDBusProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SessionDBusProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SessionDBusProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SessionDBusProxyE_t>.metaTypes,
    nullptr
} };

void SessionDBusProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SessionDBusProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->OnBatteryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->HasLidSwitchChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->HasBatteryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->BatteryPercentageChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->BatteryStatusChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 5: _t->BatteryTimeToEmptyChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 6: _t->IsHighPerformanceSupportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->PowerSavingModeEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->PowerSavingModeBrightnessDropPercentChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 9: _t->SessionActiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->notifyActionInvoked((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->timeUpdate(); break;
        case 12: _t->login1OwnerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::OnBatteryChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::HasLidSwitchChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::HasBatteryChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(double )>(_a, &SessionDBusProxy::BatteryPercentageChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(uint )>(_a, &SessionDBusProxy::BatteryStatusChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(quint64 )>(_a, &SessionDBusProxy::BatteryTimeToEmptyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::IsHighPerformanceSupportedChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::PowerSavingModeEnabledChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(uint )>(_a, &SessionDBusProxy::PowerSavingModeBrightnessDropPercentChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(bool )>(_a, &SessionDBusProxy::SessionActiveChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(uint , const QString & )>(_a, &SessionDBusProxy::notifyActionInvoked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)()>(_a, &SessionDBusProxy::timeUpdate, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SessionDBusProxy::*)(const QString & , const QString & , const QString & )>(_a, &SessionDBusProxy::login1OwnerChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->onBattery(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasLidSwitch(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasBattery(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->batteryPercentage(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->batteryStatus(); break;
        case 5: *reinterpret_cast<quint64*>(_v) = _t->batteryTimeToEmpty(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isHighPerformanceSupported(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->powerSavingModeEnabled(); break;
        case 8: *reinterpret_cast<uint*>(_v) = _t->powerSavingModeBrightnessDropPercent(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->sessionActive(); break;
        default: break;
        }
    }
}

const QMetaObject *SessionDBusProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SessionDBusProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SessionDBusProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SessionDBusProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void SessionDBusProxy::OnBatteryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SessionDBusProxy::HasLidSwitchChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SessionDBusProxy::HasBatteryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SessionDBusProxy::BatteryPercentageChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SessionDBusProxy::BatteryStatusChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SessionDBusProxy::BatteryTimeToEmptyChanged(quint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void SessionDBusProxy::IsHighPerformanceSupportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SessionDBusProxy::PowerSavingModeEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void SessionDBusProxy::PowerSavingModeBrightnessDropPercentChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void SessionDBusProxy::SessionActiveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void SessionDBusProxy::notifyActionInvoked(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void SessionDBusProxy::timeUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void SessionDBusProxy::login1OwnerChanged(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
