// SPDX-FileCopyrightText: 2026 UnionTech Software Technology Co., Ltd.
// SPDX-License-Identifier: LGPL-3.0-or-later

#pragma once

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(logAmbientBrightness)
